@@ -5,11 +5,14 @@
 # -----------------------------------
 
 # Base Imports
+# ------------
 
+import re
 import os
 import sys
 import os.path
 import pprint
+from functools import lru_cache
 
 # Reconfigurations
 # ----------------
@@ -120,43 +123,51 @@ class Node:
     Node Object
     '''
 
-    def __init__(self, name, smiles=[], smarts=[], value=None, colour=None):
+    def __init__(self, name, smiles=[], smarts=[], bit_vectors=[], value=None, colour=None):
 
         self.name = name.split('.')[0]
         self.children = []
         self.parents = []
         self.smiles = smiles
         self.smarts = smarts
+        self.bit_vector = bit_vectors
         self.state = [ self.smiles, self.smarts ]
         self.value = None
         self.colour = None
 
-    def add_parent(self, name, smiles=[], smarts=[]):
+    def add_parent(self, name, smiles=[], smarts=[], bit_vectors=[]):
 
         '''
+
         Add the Parent Node
+
         '''
 
         self.parents.append(
-            Node(name, smiles, smarts)
+            Node(name, smiles, smarts, bit_vectors)
         )
 
-    def add_child(self, name, smiles=[], smarts=[]):
+    def add_child(self, name, smiles=[], smarts=[], bit_vectors=[]):
 
         '''
+
         Add the Child Node
+
         '''
 
         self.children.append(
-            Node(name, smiles, smarts)
+            Node(name, smiles, smarts, bit_vectors)
         )
 
     def get_node_state(self):
 
         '''
+
         Get the Node State
+
         Returns:
             state (Node): Node state
+
         '''
 
         return self.state
@@ -164,7 +175,9 @@ class Node:
     def set_node_state(self):
 
         '''
+
         Set the Node State with the children and the parents.
+
         '''
 
         self.state = [ self.children, self.parents ]
@@ -172,7 +185,9 @@ class Node:
     def print_stat(self):
 
         """
+
         Print statistics of the node
+
         """
         print("Children: %s" % self.children)
         print("Parents: %s" % self.parents)
@@ -180,9 +195,12 @@ class Node:
     def set_node_value(self, value):
 
         '''
+
         Set the Node Value
+
         Arguments:
             value: value of the node you would like to set it to | Type: Anything
+
         '''
 
         self.value = value
@@ -190,10 +208,13 @@ class Node:
     def get_node_value(self):
 
         '''
+
         Get the Node Value
+
         '''
 
         return self.value
+
     # This hack is for the root node to have dummy dictionaries.
 
     @staticmethod
@@ -206,6 +227,11 @@ class Node:
 
         smarts = {}
 
+    @staticmethod
+    def get_bit_vector():
+
+        bit_vector = {}
+
 class PrintNode:
 
     __version__ = '0.0.1'
@@ -524,7 +550,8 @@ class GlobalChem(object):
             "node_value": Node(
                 root_node,
                 self.__NODES__[root_node].get_smiles(),
-                self.__NODES__[root_node].get_smarts()
+                self.__NODES__[root_node].get_smarts(),
+                self.__NODES__[root_node].get_bit_vector()
             ),
             "children": [],
             "parents": [],
@@ -580,6 +607,7 @@ class GlobalChem(object):
                 child_key,
                 self.__NODES__[ child_key ].get_smiles(),
                 self.__NODES__[ child_key ].get_smarts(),
+                self.__NODES__[ child_key ].get_bit_vector()
             )
         except:
             _ = parent_node.add_child(
@@ -595,7 +623,8 @@ class GlobalChem(object):
                 "node_value": Node(
                     child_key,
                     self.__NODES__[ child_key ].get_smiles(),
-                    self.__NODES__[ child_key ].get_smarts()
+                    self.__NODES__[ child_key ].get_smarts(),
+                    self.__NODES__[ child_key ].get_bit_vector()
                 ),
                 "children": [],
                 "parents": [],
@@ -640,7 +669,8 @@ class GlobalChem(object):
             _ = new_child_node.add_parent(
                 parent_key,
                 self.__NODES__[ parent_key ].get_smiles(),
-                self.__NODES__[ parent_key ].get_smarts()
+                self.__NODES__[ parent_key ].get_smarts(),
+                self.__NODES__[ child_key ].get_bit_vector()
             )
 
         except:
@@ -735,11 +765,14 @@ class GlobalChem(object):
     def get_node_smiles(self, node_key):
 
         '''
+
         Get the Node SMILES
+
         Arguments:
             node_key: Node key to query | Type: String
         Returns:
             smiles_dict: Dictionary of the SMILES for that object.
+
         '''
 
         node = self.network.get(node_key, None)
@@ -755,11 +788,14 @@ class GlobalChem(object):
     def get_node_smarts(self, node_key):
 
         '''
+
         Get the Node SMARTS
+
         Arguments:
             node_key: Node key to query | Type: String
         Returns:
             smarts_dict: Dictionary of the SMARTS for that object.
+
         '''
 
         node = self.network.get(node_key, None)
@@ -772,6 +808,24 @@ class GlobalChem(object):
 
         return self.__NODES__[node_key].get_smarts()
 
+    def get_node_bits(self, node_key):
+
+        '''
+
+        Get the Node Bit Vector
+
+        '''
+
+        node = self.network.get(node_key, None)
+
+        if not node:
+            raise GraphNetworkError(
+                message=f'No Node named {node_key} exists',
+                errors=''
+            )
+
+        return self.__NODES__[node_key].get_bit_vector()
+
     def initiate_deep_layer_network(self, root_node='global_chem'):
 
         '''
@@ -982,45 +1036,90 @@ class GlobalChem(object):
 
         return smarts
 
-    def get_smiles_by_iupac(self, iupac_key, return_network_path=False, return_all_network_paths=False):
+    def get_all_bits(self):
 
         '''
-        Get the SMILES by the IUPAC.
-        Arguments:
-            iupac_key (String): Key for the iupac.
-            return_network_path (Bool): whether the user wants the network path as well.
-            return_all_network_paths (Bool): return all the network paths
+
+        Fetches all the smarts in the network
+
         Returns:
-            definition (String): definiton of the iupac.
-            network_path (Dict): { definition: last network path in }
-            network_paths (Dict[List]): { definition: all_networks }
+            bits (List): Full list of SMARTS in the network.
+
         '''
 
-        network_paths = []
-        definition = ''
+        bits = []
 
         for node_key, node_value in self.__NODES__.items():
 
+            if node_key != 'global_chem' and node_key != 'common_regex_patterns':
+
+                bits.append(list(node_value.get_bit_vector().values()))
+
+        bits = sum(bits, [])
+
+        return bits
+
+    def get_smiles_by_iupac(
+            self,
+            iupac_key,
+            distance_tolerance=4,
+            return_partial_definitions=False,
+        ):
+
+        '''
+
+        Get the SMILES by the IUPAC.
+
+        Arguments:
+            iupac_key (String): Key for the iupac.
+            distance_tolerance (Int): Distance tolerance for Levenshetin Distances.
+            return_partial_definitions (Bool): Return the Levenshetin Distances between words
+        Returns:
+            definition (Dict): A definition of the IUPAC.
+
+        '''
+
+        iupac_key = iupac_key.lower()
+        iupac_key = re.sub(r'[^a-zA-Z]', '', iupac_key)
+
+        definitions = []
+        exact_definition = {}
+
+        for node_key, node_value in self.__NODES__.items():
+
+            definition = {}
+
             if node_key == 'global_chem' or node_key == 'common_regex_patterns':
                 continue
 
             network_path = node_value.__module__
             entity = node_value()
-            smiles = entity.get_smiles()
+            names = entity.get_smiles()
 
-            if iupac_key in smiles:
+            for name in names:
 
-                network_paths.append(network_path)
-                definition = smiles[iupac_key]
+                # Sanitize the Node Key
 
-        if return_network_path:
-            return { definition: network_paths[-1] }
+                name = name.lower()
+                name = re.sub(r'[^a-zA-Z]', '', name)
 
-        elif return_all_network_paths:
-            return { definition: return_all_network_paths }
+                distance = self.levenshtein_distance(iupac_key, name)
 
+                if 0 <= distance <= distance_tolerance:
+
+                    definition[iupac_key] = names[name]
+                    definition['network_path'] = network_path[-1]
+                    definition['levenshtein_distance'] = distance
+
+                    if distance == 0:
+                        exact_definition = definition
+
+                definitions.append(definition)
+
+        if return_partial_definitions:
+            return definitions
         else:
-            return definition
+            return exact_definition
 
     def set_node_value(self, node_key, value):
 
@@ -1259,3 +1358,36 @@ class GlobalChem(object):
                 out_file.write(f'{key}\t{value}\t{node_name}\t{category}\t{tree_path}\n')
 
         out_file.close()
+
+    @staticmethod
+    def levenshtein_distance(iupac_a, iupac_b):
+
+        '''
+
+        This function will calculate the levenshtein distance between two IUPAC strings
+
+        params:
+            a (String) : The first string you want to compare
+            b (String) : The second string you want to compare
+
+        returns:
+            This function will return the distnace between string a and b.
+
+        '''
+
+        @lru_cache(None)
+        def min_dist(string_1, string_2):
+
+            if string_1 == len(iupac_a) or string_2 == len(iupac_b):
+                return len(iupac_a) - string_1 + len(iupac_b) - string_2
+
+            if iupac_a[string_1] == iupac_b[string_2]:
+                return min_dist(string_1 + 1, string_2 + 1)
+
+            return 1 + min(
+                min_dist(string_1, string_2 + 1),
+                min_dist(string_1 + 1, string_2),
+                min_dist(string_1 + 1, string_2 + 1),
+            )
+
+        return min_dist(0, 0)
\ No newline at end of file
